<?php

namespace LQDN\Tests\Functional\Handler;

use LQDN\Command\AddressCreateCommand;
use LQDN\Command\AddressDeleteCommand;
use LQDN\Command\AddressUpdateCommand;
use LQDN\Handler\AddressHandler;
use LQDN\Tests\Functional\FunctionalTest;

class AddressHandlerTest extends FunctionalTest
{
    public function testAddressUsedDelete()
    {
        $this->assertTrue($this->addressExists(1));

        $this->setExpectedException('LQDN\Exception\AddressUsedException');
        $this->container['command_handler']->handle(new AddressDeleteCommand(1, 1));
    }

    public function testAdressDelete()
    {
        $this->assertTrue($this->addressExists(2));

        $this->container['command_handler']->handle(new AddressDeleteCommand(2, 1));

        $this->assertFalse($this->addressExists(2));
    }

    public function testAddressUpdate()
    {
        $this->assertTrue($this->addressExists(1));
        $this->assertTrue($this->isDefaultAddress(1));
        $this->assertTrue($this->addressExists(2));
        $this->assertFalse($this->isDefaultAddress(2));

        $this->container['command_handler']->handle(new AddressUpdateCommand(2, 1, 'updated', '6 rue Ménars', '', 75002, 'Paris', 'France', 'IDF', true));

        $this->assertFalse($this->isDefaultAddress(1));
        $this->assertTrue($this->isDefaultAddress(2));
    }

    public function testAddressCreateWithSpecialChars()
    {
        $this->assertTrue($this->addressExists(1));
        $this->assertTrue($this->isDefaultAddress(1));

        $this->container['command_handler']->handle(new AddressCreateCommand(1, 'New address', 'updated', '6 rue Ménars', '', 22000, 'Saint Brieuc', 'France', 'Côtes d\'Armor', true));

        $expectedAddress = [
            'nom' => 'New address',
            'adresse' => '6 rue Ménars',
            'adresse2' => '',
            'codepostal' => '22000',
            'ville' => 'Saint Brieuc',
            'etat' => 'Côtes d\'Armor',
            'pays' => 'France',
            'alias' => 'updated',
            'user_id' => '1',
            'defaut' => '1',
        ];

        $latestAddress = $this->getLatestAddress();
        unset($latestAddress['id']);

        $this->assertFalse($this->isDefaultAddress(1));
        $this->assertSame($expectedAddress, $latestAddress);
    }

    /**
     * Check if an address exists in DB.
     *
     * @param int $id
     *
     * @return bool
     */
    private function addressExists($id)
    {
        return (bool) $this->container['db']->fetchColumn("SELECT 1 FROM adresses WHERE id = $id");
    }

    /**
     * Check if an address is marked as default.
     *
     * @param int $id
     *
     * @return bool
     */
    private function isDefaultAddress($id)
    {
        return (bool) $this->container['db']->fetchColumn("SELECT 1 FROM adresses WHERE id = $id and defaut=true");
    }

    /**
     * Retrieve latest address.
     *
     * @return []
     */
    private function getLatestAddress()
    {
        return $this->container['db']->fetchAssoc("SELECT * FROM adresses ORDER BY id DESC LIMIT 1");
    }
}
